# Imputation using MICE (multiple imputation
# with chained equations)

library(mice)

# very basic imputation example

rm(list=ls())
set.seed(123897)

# create non-missing data set
x.t<-runif(200)
z.t<-runif(200)
y.t<-x.t+z.t+rnorm(200, sd=0.5)

# induce 10% missingness in each variable
miss.x<-rbinom(200, 1, prob=0.9)
miss.z<-rbinom(200, 1, prob=0.9)
miss.y<-rbinom(200, 1, prob=0.9)

x<-ifelse(miss.x==1,x.t,NA)
z<-ifelse(miss.z==1, z.t, NA)
y<-ifelse(miss.y==1, y.t, NA)

# bind the data frame with missingness
dat<-data.frame(x,z,y)

# create 5 MI data sets 
mice(dat, m=5, maxit=0)$method
mi.dat<-mice(dat, m=5, maxit=20)

# visually assess that
bwplot(mi.dat)
densityplot(mi.dat)
xyplot(mi.dat, x+z~.imp)

# fit model with and without imputation
fit<-with(mi.dat, lm(y~x+z))
summary(pool(fit))
summary(lm(y~x+z, data=dat))







# basic imputation, with more missingness

rm(list=ls())
set.seed(123897)

# create non-missing data set
x.t<-runif(200)
z.t<-runif(200)
y.t<-x.t+z.t+rnorm(200, sd=0.5)

# induce 10% missingness in each variable
miss.x<-rbinom(200, 1, prob=0.5)
miss.z<-rbinom(200, 1, prob=0.5)
miss.y<-rbinom(200, 1, prob=0.5)

x<-ifelse(miss.x==1,x.t,NA)
z<-ifelse(miss.z==1, z.t, NA)
y<-ifelse(miss.y==1, y.t, NA)

# bind the data frame with missingness
dat<-data.frame(x,z,y)

# create 5 MI data sets 
mi.dat<-mice(dat, m=10, maxit=40)

# visually assess that
bwplot(mi.dat)
densityplot(mi.dat)
xyplot(mi.dat, x+z+y~.imp)

# fit model with and without imputation
fit<-with(mi.dat, lm(y~x+z))
summary(pool(fit))
summary(lm(y~x+z, data=dat))









# imputation of a model with interaction

rm(list=ls())
set.seed(0241721)

# create non-missing data set
x.t<-runif(500)
z.t<-runif(500)
y.t<-x.t+z.t+x.t*z.t+rnorm(500, sd=0.5)

# induce 10% missingness in each variable
miss.x<-rbinom(500, 1, prob=0.9)
miss.z<-rbinom(500, 1, prob=0.9)
miss.y<-rbinom(500, 1, prob=0.9)

x<-ifelse(miss.x==1,x.t,NA)
z<-ifelse(miss.z==1, z.t, NA)
xz<-x*z
y<-ifelse(miss.y==1, y.t, NA)

# bind the data frame with missingness
dat<-data.frame(x,z,xz,y)

# initialize the mi object
mi.init<-mice(dat, m=1, maxit=0)

# create the passive imputation 
meth<-mi.init$meth
meth["xz"]<-"~I(x*z)"
pred<-mi.init$pred
# don't use xz to predict x or z (obviously!)
pred[c("x","z"),"xz"]<-0

# generate the imputed data sets
mi.dat<-mice(dat, m=20, maxit=40, meth=meth, pred=pred)

# visually assess that
bwplot(mi.dat)
densityplot(mi.dat)
xyplot(mi.dat, x+z+xz+y~.imp)

# fit model with and without imputation
fit<-with(mi.dat, lm(y~x+z+xz))
summary(pool(fit))
summary(lm(y~x+z+xz, data=dat))









# binary independent variable

rm(list=ls())
set.seed(123897)

# create non-missing data set
x.t<-runif(200)
z.t<-factor(ifelse(runif(200)>0.5, 1, 0))
y.t<-x.t+as.numeric(z.t)+rnorm(200, sd=0.5)

# induce 10% missingness in each variable
miss.x<-rbinom(200, 1, prob=0.9)
miss.z<-rbinom(200, 1, prob=0.9)
miss.y<-rbinom(200, 1, prob=0.9)

x<-ifelse(miss.x==1,x.t,NA)
z<-factor(ifelse(miss.z==1, z.t, NA)-1)
y<-ifelse(miss.y==1, y.t, NA)

# bind the data frame with missingness
dat<-data.frame(x,z,y)

# create 5 MI data sets 
mi.init<-mice(dat, m=1, maxit=0)
mi.init$meth
mi.dat<-mice(dat, m=5, maxit=20)

# visually assess that
plot(mi.dat)
bwplot(mi.dat)
densityplot(mi.dat)

# fit model with and without imputation
fit<-with(mi.dat, lm(y~x+z))
summary(pool(fit))
summary(lm(y~x+z, data=dat))







# imputation using Bayesian data augmentation

rm(list=ls())

library(arm)
library(BRugs)
library(foreign)
library(mcmcplots)
library(R2WinBUGS)

# without imputation first

dta<-read.dta(file="anes-data.dta")
dta.2008<-subset(dta, year==2008)
attach(dta.2008)

sum(is.na(gaytherm))
sum(is.na(partyid))
sum(is.na(churchattend))
sum(is.na(age))

dta.2008.s<-subset(dta.2008, is.na(partyid)==F & is.na(churchattend)==F)
detach(dta.2008)

sum(is.na(dta.2008.s$gaytherm))
sum(is.na(dta.2008.s$partyid))
sum(is.na(dta.2008.s$churchattend))
sum(is.na(dta.2008.s$age))

dta.2008.s$churchattend<-as.numeric(dta.2008.s$churchattend)
dta.2008.s$partyid<-as.numeric(dta.2008.s$partyid)
dta.2008.s$age<-as.numeric(dta.2008.s$age)
attach(dta.2008.s)

N<-dim(dta.2008.s)[1]

age.ind<-c()
o<-order(unique(age))
for(i in 1:length(year)){
  age.ind[i]<-ifelse(is.na(age[i]), NA, which(unique(age)[o]==age[i]))
}

data<- list("N", "gaytherm", "age.ind","churchattend", "partyid")
inits <- function() {list (b0=0, b.age = 0, b.church = 0, b.party=0, tau.y = 1, tau.I=1)}
parameters<- c("b0", "b.age", "b.church", "b.party", "tau.y")
regression.sim<-bugs(data, inits, parameters.to.save=parameters, model.file="anes-imputation.bug", n.chains=1, n.iter=10000, n.burnin=1000, n.thin=1, debug=T, program="WinBUGS")
caterplot(regression.sim, parms="b.age")

summary(lm(gaytherm~age.ind+churchattend+partyid))